#!/usr/bin/python
import os
import tempfile
import argparse
import socket
import shutil

from zope.interface import implements
from twisted.internet import reactor, ssl
from twisted.web import server, resource, guard, static
from twisted.cred.portal import IRealm, Portal
from twisted.protocols.ftp import FTPFactory, FTPRealm
from twisted.cred.checkers import AllowAnonymousAccess
from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse as InMemoryDB


def get_listen_ip():
    return socket.gethostbyname(socket.gethostname())


def create_work_area():
    return tempfile.mkdtemp()


def add_test_file(pub_dir):
    test_file = os.path.join(pub_dir, 'helloworld')
    f = open(test_file, 'w')
    f.write('If it aint tip top then it aint tip top\n')
    f.close()


def tidyup(work_dir):
    shutil.rmtree(work_dir)


class SimpleRealm(object):
    implements(IRealm)

    def requestAvatar(self, avatarId, mind, *interfaces):
        if resource.IResource in interfaces:
            return resource.IResource, root, lambda: None
        raise NotImplementedError()


def start_http_server(work_dir, port, proto, **users):
    add_test_file(work_dir)
    if users:
        checkers = [InMemoryDB(**users)]
        for user in users:
            print "%s://%s:%s@%s:%s/" % (proto, user, users[user],
                                         get_listen_ip(), port)
    else:
        checkers = [AllowAnonymousAccess()]
        print "%s://%s:%s/" % (proto, get_listen_ip(), port)

    wrapper = guard.HTTPAuthSessionWrapper(
        Portal(SimpleRealm(), checkers),
        [guard.BasicCredentialFactory('test realm')])
    if proto == 'https':
        sslContext = ssl.DefaultOpenSSLContextFactory(
            'privkey.pem',
            'cacert.pem',
        )
        reactor.listenSSL(port, server.Site(resource=wrapper),
                          contextFactory=sslContext)
    else:
        reactor.listenTCP(port, server.Site(resource=wrapper))
    try:
        reactor.run()
    finally:
        tidyup(work_dir)


def start_ftp_server(work_dir, port, **users):
    if users:
        for user in users:
            print "ftp://%s:%s@%s:%s/" % (user, users[user],
                                          get_listen_ip(), port)
            os.mkdir(os.path.join(work_dir, user))
            add_test_file(os.path.join(work_dir, user))
    else:
        add_test_file(work_dir)
        print "ftp://%s:%s/" % (get_listen_ip(), port)

    p = Portal(FTPRealm(work_dir, userHome=work_dir),
               (AllowAnonymousAccess(),
                InMemoryDB(**users),
                )
               )

    f = FTPFactory(p)

    try:
        reactor.listenTCP(port, f)
        reactor.run()
    finally:
        tidyup(work_dir)

parser = argparse.ArgumentParser()
parser.add_argument("listen_proto",
                    help="Protocol server to create http, https, ftp")
parser.add_argument("--port", type=int, help="Port to listen on")
parser.add_argument('--anon', action='store_true', default=False,
                    help="Setup anonymous access")

args = parser.parse_args()

default_ports = {'ftp': 2121, 'http': 8080, 'https': 4343}
if args.port:
    listen_port = args.port
else:
    listen_port = default_ports[args.listen_proto]
if args.anon:
    creds = {}
else:
    creds = {'bob': 'bob'}

work_area = create_work_area()
root = static.File(work_area)

if args.listen_proto == 'http':
    start_http_server(work_area, listen_port, args.listen_proto, **creds)
elif args.listen_proto == 'https':
    start_http_server(work_area, listen_port, args.listen_proto, **creds)
elif args.listen_proto == 'ftp':
    start_ftp_server(work_area, listen_port, **creds)
